import json
import os
import pandas as pd
import pathlib


# 将csv中的实体和关系信息转化为cnschema形式
def csv2jsonLD(property_csv: pathlib.Path, relation_csvs: dict, property_dict: dict, relation_dicts: dict):
    property = pd.read_csv(property_csv)

    for i, row in property.iterrows():
        graph_row = {}
        graph_row["@id"] = domain_url + str(row["ID"])
        graph_row["@type"] = domain_url + str(row["LABEL"])
        graph_row["id"] = str(row.ID)
        for item in property_dict.keys():
            graph_row[property_dict[item]] = str(row[item])
        graph_row["relationships"] = {}
        for relation in relation_csvs.keys():
            relation_df = pd.read_csv(relation_csvs[relation])
            index = 0
            for start_id in relation_df["START_ID"]:
                if (str(start_id) == str(row["ID"])):
                    graph_row["relationships"][relation_dicts[relation][relation_df.iloc[index]["type"]]] = {
                        "@id": domain_url + relation_df.iloc[index]["END_ID"],
                        "@type": relation
                    }

                index += 1
        global_data["graph"].append(graph_row)


if __name__ == "__main__":
    domain_url = "http://openkg.cn/dataset/threekingdom_zju/"
    root = pathlib.Path("../")
    result_jsonLD = root / "cnSchema/threekingdom.jsonLd"
    global_data = {"context": {"@vocab": "http://cnschema.org/"}, "graph": []}

    # csv文件声明
    figure_csv, country_csv = root / "figure.csv", root / "country.csv"
    figure2figure_csv, figure2country_csv = root / "figure2figure.csv", root / "figure2country.csv"

    # 项目中英翻译
    figure2figure_dict = {'正室之父': "fatherOfLegalWife", '正室之母': "motherOfLegalWife", '養母': "adoptiveMother",
                          '繼母': "stepmother", '繼室之父': "fatherOfSecondWife", '女': "daughter", '庶兄弟': "commonBrother",
                          '儿女': "children", '妻之母': "motherOfWife", '繼子': "secondSon",
                          '繼室': "secondWife",
                          '異母兄弟': "halfBrother", '妻': "wife", '子': "son", '嫡兄弟': "legitimateBrother",
                          '養子': "adoptedSon", '庶女': "daughterOfConcubine", '亲属': "relatives",
                          '元配之母': "motherOfFirstWife", '母親': "mother",
                          '兄弟': "brother",
                          '庶姊妹': "commonSister", '元配': "firstWife", '元配之父': "fatherOfFirstWife",
                          '同胞姊妹': "siblingSister", '嫡姊妹': "directSister", '正室': "legalWife", '姊妹': "sister",
                          '主君': "lord", '師承': "teather", '傳人': "student", '妃嫔': "concubineOfEmperor",
                          '前任': "predecessor", '君主': "king", '嫡女': "legitimateDaughter", '繼父': "stepfather",
                          '父親': "father", '側室': "concubine", '養女': "adoptedDaughter",
                          '同胞兄弟': "sablingBrother", '養父': "adoptedFather", '宰相': "primeMinister",
                          '其他親屬': "otherRelatives", '異母姊妹': "halfSisterF", '王后': "queen",
                          '庶子': "sonOfConcubine", '嫡母': "legitimateMother", '妻之父': "fatherOfWife",
                          '嫡子': "legitimateSon", '配偶': "spouse"}
    figure_dict = {'位階': "rank", '祖籍': "ancestralHome", '廟號': "temple", '姓': "lastName", '京城': "capital", '封地': "manor",
                   '號': "pseudonym", '時代': "time", '简介': "introduction", '政权': "regime", '逝世': "death",
                   '墓地': "cemetery", '頭銜': "title", '朝代': "dynasty",
                   '出生': "born", '祠廟': "shrine", '種族': "nation", '其他名號': "otherPseudonym", '继任': "next", '尊號': "honor",
                   '籍贯': "hometown", '族裔': "ethnicity", '別名': "otherName", '名': "firstname",
                   '昵称': "nickname", '本籍': "nativeHometown", '統治': "rule", '墓葬': "Tomb", '字': "styleName", '宗派': "sect",
                   '婚年': "wedding", '职业': "job", '姓名': "name", '爵位': "knighthood", '出身地': "placeOfBirth",
                   '職官': "officialPlace", '封爵': "knight", '居住地': "livingPlace",
                   '世系': "desent", '安葬': "burial", '本貫': "honkan", '氏族': "clan", '封號': "givenTitle",
                   '王朝': "rulingDynasty", '呼號': "sign", '登基': "enthrone", '初名': "originalName", '諡號': "posthumousName",
                   '子嗣': "sons"}
    country_dict = {'建立 ': "establish", '国君姓氏': "lastNameOfKing", '简介': "introduction", '宗教': "religion",
                    '人口': "population", '通币': "money", '国名': "name", '历史时期': "historicalPeriod", '今属于': "nowBelongTo",
                    '政府': "government", '常用语': "language",
                    '首都': "capital", '灭亡 ': "perish", '君主': "king"}
    figure2country_dict = {"国家": "country"}

    csv2jsonLD(figure_csv, {"figure": figure2figure_csv, "country": figure2country_csv}, figure_dict,
               {"figure": figure2figure_dict, "country": figure2country_dict})
    csv2jsonLD(country_csv, {}, country_dict, {})

    with open(result_jsonLD, 'w', encoding='utf-8') as f:
        json.dump(global_data, f, ensure_ascii=False, indent=4)
