import json
import csv
import pathlib


# 从提供的json文件中抽取出实体基本信息
def json2properties(json_file: pathlib.Path, csv_file: pathlib.Path, properties_list: list, label: str, prefix: str):
    with open(json_file, 'rb') as f:
        entities = json.load(f)
        # keys = set()
        # for entity in entities:
        #     for key in entity.keys():
        #         keys.add(key)
        # print(keys)
        with open(csv_file, 'w', encoding='utf-8', newline="") as c:
            begin =  "ID"
            end = "LABEL"
            csv_writer = csv.writer(c)
            csv_writer.writerow([begin] + properties_list + [end])
            index = 0
            for entity in entities:
                data = [prefix + str(index)]
                for p in properties_list:
                    try:
                        data.append(str(entity[p]))
                    except:
                        data.append("N/A")
                data.append(label)
                csv_writer.writerow(data)
                index += 1


def query(jsons, name, key):
    res = 0
    for json in jsons:
        if (json[key] == name):
            return res
        res += 1
    return -1


# 从给定的json文件中抽取实体之间的关系
def json2realtion(relation_json_file: pathlib.Path, property_json_file: pathlib.Path, csv_file: pathlib.Path,
                  relationss_list: list, query_key: str, prefix_start: str, prefix_end: str):
    headers = ["END_ID", "type", "START_ID"]
    with open(relation_json_file, 'rb') as rj:
        relations = json.load(rj)
        with open(property_json_file, 'rb') as pj:
            properties = json.load(pj)
            with open(csv_file, "w", encoding="utf-8", newline="") as c:
                csv_writer = csv.writer(c)
                csv_writer.writerow(headers)
                index = 0
                for relation in relations:
                    start_id = index
                    for r in relationss_list:
                        try:
                            type = r
                            name = relation[r]
                            for name1 in name.split('、'):
                                for name2 in name1.split('，'):
                                    end_id = query(properties, name2, query_key)
                                    if (end_id == -1): continue
                                    data = [prefix_end + str(end_id), type, prefix_start + str(start_id)]
                                    csv_writer.writerow(data)
                        except:
                            continue

                    index += 1


if __name__ == '__main__':
    entities = ["figure", "country"]
    root = pathlib.Path("./")
    figure_json, figure_csv = root / "result.json", root / "figure.csv"
    figure2figure_csv, figure2country_csv = root / "figure2figure.csv", root / "figure2country.csv"
    country_json, country_csv = root / "result_country.json", root / "country.csv"
    country2figure_csv = root / "country2figure.csv"

    # 实体属性列表
    figure_properties_list = ['位階', '祖籍', '廟號', '姓', '京城', '封地', '號', '時代', '简介', '政权', '逝世', '墓地', '頭銜', '朝代',
                              '出生', '祠廟', '種族', '其他名號', '继任', '尊號', '籍贯', '族裔', '別名', '名',
                              '昵称', '本籍', '統治', '墓葬', '字', '宗派', '婚年', '职业', '姓名', '爵位', '出身地', '職官', '封爵', '居住地',
                              '世系', '安葬', '本貫', '氏族', '封號', '王朝', '呼號', '登基', '初名', '諡號', '子嗣']
    country_properties_list = ['建立 ', '国君姓氏', '简介', '宗教', '人口', '通币', '国名', '历史时期', '今属于', '政府', '常用语',
                               '首都', '灭亡 ', '君主']

    # 实体关系列表
    figure2country_relations_list = ['国家']
    figure2figure_relations_list = ['正室之父', '正室之母', '養母', '繼母', '繼室之父', '女', '庶兄弟', '儿女', '妻之母', '繼子',
                                    '繼室', '異母兄弟', '妻', '子', '嫡兄弟', '養子', '庶女', '亲属', '元配之母', '母親',
                                    '兄弟', '庶姊妹', '元配', '元配之父', '同胞姊妹', '嫡姊妹', '正室', '姊妹', '主君', '師承', '傳人', '妃嫔',
                                    '前任', '君主', '嫡女', '繼父', '父親', '側室', '養女', '同胞兄弟', '養父', '宰相', '其他親屬', '異母姊妹', '王后',
                                    '庶子', '嫡母', '妻之父', '嫡子', '配偶']


    json2properties(figure_json, figure_csv, figure_properties_list, "figure", "figure#")
    json2realtion(figure_json, figure_json, figure2figure_csv, figure2figure_relations_list, "姓名", "figure#", "figure#")
    json2properties(country_json, country_csv, country_properties_list, "country", "country#")
    json2realtion(figure_json, country_json, figure2country_csv, figure2country_relations_list, "国名", "figure#",
                  "country#")
